<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Block;

use Amasty\PriceHistory\Model\ConfigProvider;
use Amasty\PriceHistory\Model\Frontend\DisplayValidator;
use Amasty\PriceHistory\Model\PriceHistory\ProductLowestPriceProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Customer\Model\Context as CustomerContext;
use Magento\Framework\App\Http\Context;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Framework\View\Element\Template;

class LowestPrice extends Template
{
    /**
     * @var string
     */
    protected $_template = 'Amasty_PriceHistory::lowest_price.phtml';

    /**
     * @var DisplayValidator
     */
    private DisplayValidator $displayValidator;

    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var ProductLowestPriceProvider
     */
    private ProductLowestPriceProvider $lowestPriceProvider;

    /**
     * @var ProductInterface|null
     */
    private ?ProductInterface $product = null;

    /**
     * @var Context
     */
    private Context $httpContext;

    /**
     * @var PriceCurrencyInterface
     */
    private PriceCurrencyInterface $priceCurrency;

    public function __construct(
        Template\Context $context,
        DisplayValidator $displayValidator,
        ConfigProvider $configProvider,
        ProductLowestPriceProvider $lowestPriceProvider,
        Context $httpContext,
        PriceCurrencyInterface $priceCurrency,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->displayValidator = $displayValidator;
        $this->configProvider = $configProvider;
        $this->lowestPriceProvider = $lowestPriceProvider;
        $this->httpContext = $httpContext;
        $this->priceCurrency = $priceCurrency;
    }

    public function getDisplayText(): string
    {
        $productId = (int)$this->getProduct()->getId();
        $period = $this->configProvider->getDisplayPeriod((int)$this->_storeManager->getStore()->getId());
        $customerGroupId = (int)$this->httpContext->getValue(CustomerContext::CONTEXT_GROUP);

        $price = $this->lowestPriceProvider->get($productId, $period, $customerGroupId);
        if (!$price) {
            return '';
        }

        $formattedPrice = $this->priceCurrency->convertAndFormat($price, false);

        return __('The lowest price from last %1 days: %2', $period, $formattedPrice)->render();
    }

    public function setProduct(ProductInterface $product): void
    {
        $this->product = $product;
    }

    public function getProduct(): ?ProductInterface
    {
        return $this->product;
    }

    public function setAdditionalClasses(string $class): void
    {
        $this->setData('additional_classes', $class);
    }

    public function getAdditionalClasses(): string
    {
        return (string)$this->getData('additional_classes');
    }

    protected function _toHtml(): string
    {
        if (!$this->canShow()) {
            return '';
        }

        return parent::_toHtml();
    }

    private function canShow(): bool
    {
        if ($product = $this->getProduct()) {
            return $this->displayValidator->validate($product);
        }

        return false;
    }
}
