<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Block;

use Amasty\PriceHistory\Model\ConfigProvider;
use Amasty\PriceHistory\Model\Frontend\LowestPriceRenderer;
use Amasty\PriceHistory\Model\OptionSource\DisplayPrices;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\Template;

class ProductLowestPriceWrapper extends Template
{
    public const PRODUCT_PAGE_CLASS = 'product-page';

    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var LowestPriceRenderer
     */
    private LowestPriceRenderer $lowestPriceRenderer;

    /**
     * @var Registry
     */
    private Registry $registry;

    public function __construct(
        Template\Context $context,
        ConfigProvider $configProvider,
        LowestPriceRenderer $lowestPriceRenderer,
        Registry $registry,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->configProvider = $configProvider;
        $this->lowestPriceRenderer = $lowestPriceRenderer;
        $this->registry = $registry;
    }

    protected function _toHtml(): string
    {
        if ($this->configProvider->getDisplayPrices() !== DisplayPrices::NONE) {
            $product = $this->registry->registry('product');
            if ($product) {
                return $this->lowestPriceRenderer->render($product, self::PRODUCT_PAGE_CLASS);
            }
        }

        return parent::_toHtml();
    }
}
