<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Controller\Adminhtml\PriceChange;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\Auth\Session;
use Magento\Store\Model\StoreManagerInterface;

class Clear extends Action implements \Magento\Framework\App\Action\HttpGetActionInterface
{
    public const ADMIN_RESOURCE = MassDelete::ADMIN_RESOURCE;

    /**
     * @var Session
     */
    private Session $backendSession;

    /**
     * @var PriceHistory
     */
    private PriceHistory $priceHistoryResource;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    public function __construct(
        Context $context,
        Session $backendSession,
        PriceHistory $priceHistoryResource,
        StoreManagerInterface $storeManager
    ) {
        parent::__construct($context);
        $this->backendSession = $backendSession;
        $this->priceHistoryResource = $priceHistoryResource;
        $this->storeManager = $storeManager;
    }

    public function execute()
    {
        try {
            if ($websiteIds = $this->getAvailableWebsiteIds()) {
                $this->priceHistoryResource->cleanByWebsiteIds($websiteIds);
            } else {
                $this->priceHistoryResource->clean();
            }
            $this->messageManager->addSuccessMessage(__('Actions Log has been successfully cleared.'));
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        }

        return $this->resultRedirectFactory->create()->setRefererUrl();
    }

    private function getAvailableWebsiteIds(): array
    {
        $userRole = $this->backendSession->getUser()->getRole();
        if ($userRole->getData('gws_is_all')) {
            return [];
        }

        if ($userRole->hasData('gws_stores')) {
            return $this->getWebsitesByStoreIds($userRole->getData('gws_stores') ?? []);
        }

        return [];
    }

    private function getWebsitesByStoreIds(array $storeIds): array
    {
        $result = [];
        foreach ($storeIds as $storeId) {
            $result[] = $this->storeManager->getStore($storeId)->getWebsiteId();
        }

        return $result;
    }
}
