<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model;

use Amasty\Base\Model\ConfigProviderAbstract;

class ConfigProvider extends ConfigProviderAbstract
{
    /**
     * @var string
     */
    protected $pathPrefix = 'price_history/';

    public const ENABLED = 'general/enable';
    public const AUTO_CLEAN = 'general/auto_clean';
    public const AUTO_CLEAN_DAYS = 'general/auto_clean_days';
    public const DISPLAY_PRICES = 'general/display_prices';
    public const DISPLAY_PERIOD = 'general/display_period';
    public const DISPLAY_MODE = 'general/display_mode';
    public const RESTRICTED_CUSTOMER_GROUPS = 'general/restricted_customer_groups';

    public function isEnabled(?int $storeId = null): bool
    {
        return $this->isSetFlag(self::ENABLED, $storeId);
    }

    public function isAutoClean(): bool
    {
        return $this->isSetFlag(self::AUTO_CLEAN);
    }

    public function getAutoCleanDays(): int
    {
        return (int)$this->getValue(self::AUTO_CLEAN_DAYS);
    }

    public function getDisplayPrices(?int $storeId = null): int
    {
        return (int)$this->getValue(self::DISPLAY_PRICES, $storeId);
    }

    public function getDisplayPeriod(?int $storeId = null): int
    {
        return (int)$this->getValue(self::DISPLAY_PERIOD, $storeId);
    }

    public function getDisplayMode(?int $storeId = null): int
    {
        return (int)$this->getValue(self::DISPLAY_MODE, $storeId);
    }

    /**
     * @param int|null $storeId
     * @return string[]
     */
    public function getRestrictedCustomerGroups(?int $storeId = null): array
    {
        return explode(',', (string)$this->getValue(self::RESTRICTED_CUSTOMER_GROUPS, $storeId));
    }
}
