<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\Frontend;

use Amasty\PriceHistory\Block\LowestPrice;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\View\LayoutInterface;

class LowestPriceRenderer
{
    /**
     * @var LayoutInterface
     */
    private LayoutInterface $layout;

    /**
     * @var LowestPrice|null
     */
    private ?LowestPrice $blockTemplate = null;

    public function __construct(
        LayoutInterface $layout
    ) {
        $this->layout = $layout;
    }

    public function render(ProductInterface $product, string $additionalClasses = ''): string
    {
        $block = $this->getLowestPriceBlock();
        $block->setProduct($product);
        $block->setAdditionalClasses($additionalClasses);

        return $block->toHtml();
    }

    private function getLowestPriceBlock(): LowestPrice
    {
        if ($this->blockTemplate === null) {
            $this->blockTemplate = $this->layout->createBlock(LowestPrice::class);
        }

        return $this->blockTemplate;
    }
}
