<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory;

use Amasty\PriceHistory\Api\Data\PriceHistoryExtensionInterface;
use Amasty\PriceHistory\Api\Data\PriceHistoryInterface;
use Magento\Framework\Model\AbstractExtensibleModel;

class PriceHistory extends AbstractExtensibleModel implements PriceHistoryInterface
{
    public const ID = 'id';
    public const PRODUCT_ID = 'product_id';
    public const DATE = 'date';
    public const CUSTOMER_GROUP_ID = 'customer_group_id';
    public const WEBSITE_ID = 'website_id';
    public const LATEST_PRICE = 'latest_price';
    public const NEW_PRICE = 'new_price';

    /**
     * @var string
     */
    protected $_eventPrefix = 'amasty_price_history';

    /**
     * @var string
     */
    protected $_eventObject = 'price_history';

    protected function _construct()
    {
        $this->_init(ResourceModel\PriceHistory::class);
        $this->setIdFieldName(self::ID);
    }

    public function getProductId(): int
    {
        return (int)$this->_getData(self::PRODUCT_ID);
    }

    public function setProductId(int $productId): void
    {
        $this->setData(self::PRODUCT_ID, $productId);
    }

    public function getDate(): string
    {
        return (string)$this->_getData(self::DATE);
    }

    public function setDate(string $date): void
    {
        $this->setData(self::DATE, $date);
    }

    public function getCustomerGroupId(): int
    {
        return (int)$this->_getData(self::CUSTOMER_GROUP_ID);
    }

    public function setCustomerGroupId(int $customerGroupId): void
    {
        $this->setData(self::CUSTOMER_GROUP_ID, $customerGroupId);
    }

    public function getWebsiteId(): int
    {
        return (int)$this->_getData(self::WEBSITE_ID);
    }

    public function setWebsiteId(int $websiteId): void
    {
        $this->setData(self::WEBSITE_ID, $websiteId);
    }

    public function getLatestPrice(): ?float
    {
        return $this->hasData(self::LATEST_PRICE)
            ? (float)$this->_getData(self::LATEST_PRICE)
            : null;
    }

    public function setLatestPrice(float $latestPrice): void
    {
        $this->setData(self::LATEST_PRICE, $latestPrice);
    }

    public function getNewPrice(): float
    {
        return (float)$this->_getData(self::NEW_PRICE);
    }

    public function setNewPrice(float $newPrice): void
    {
        $this->setData(self::NEW_PRICE, $newPrice);
    }

    public function getExtensionAttributes()
    {
        return $this->_getExtensionAttributes();
    }

    public function setExtensionAttributes(PriceHistoryExtensionInterface $extensionAttributes = null): void
    {
        $this->_setExtensionAttributes($extensionAttributes);
    }
}
