<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory;

use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\ObjectManagerInterface;

class PriceHistoryProcessorFactory
{
    public const INDEXER_PROCESSOR = 'index';

    /**
     * @var ObjectManagerInterface
     */
    private ObjectManagerInterface $objectManager;

    /**
     * @var string[] [class_names]
     */
    private array $processors;

    public function __construct(
        ObjectManagerInterface $objectManager,
        array $processors = []
    ) {
        $this->objectManager = $objectManager;
        $this->processors = $processors;
    }

    /**
     * @param string $type
     * @param array $args
     *
     * @return Processor\PriceHistoryProcessorInterface
     * @throws NotFoundException
     * @throws \InvalidArgumentException
     */
    public function create(string $type, array $args = []): Processor\PriceHistoryProcessorInterface
    {
        if (!isset($this->processors[$type])) {
            throw new NotFoundException(
                __('The "%1" price history processor type isn\'t defined. Verify the processor and try again.', $type)
            );
        }

        $processor = $this->objectManager->create($this->processors[$type], $args);
        if (!$processor instanceof Processor\PriceHistoryProcessorInterface) {
            throw new \InvalidArgumentException(
                'The price history processor instance "' . $type . '" must implement '
                . Processor\PriceHistoryProcessorInterface::class
            );
        }

        return $processor;
    }
}
