<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\Processor;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\FillLatestPriceTable;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\UpdatePriceHistoryTable;

/**
 * Logs price changes depend on catalog_product_index_price table changes
 */
class IndexProcessor implements PriceHistoryProcessorInterface
{
    /**
     * @var FillLatestPriceTable
     */
    private FillLatestPriceTable $fillLatestPriceTable;

    /**
     * @var UpdatePriceHistoryTable
     */
    private UpdatePriceHistoryTable $updatePriceHistoryTable;

    public function __construct(
        FillLatestPriceTable $fillLatestPriceTable,
        UpdatePriceHistoryTable $updatePriceHistoryTable
    ) {
        $this->fillLatestPriceTable = $fillLatestPriceTable;
        $this->updatePriceHistoryTable = $updatePriceHistoryTable;
    }

    public function initialize(?array $ids): void
    {
        $this->fillLatestPriceTable->execute($ids);
    }

    public function execute(?array $ids): void
    {
        $this->updatePriceHistoryTable->execute($ids);
    }
}
