<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory as PriceHistoryResource;
use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Store\Model\StoreManagerInterface;

class ProductLowestPriceProvider
{
    /**
     * @var PriceHistoryResource
     */
    private PriceHistoryResource $priceHistoryResource;

    /**
     * @var TimezoneInterface
     */
    private TimezoneInterface $localeDate;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    public function __construct(
        PriceHistoryResource $priceHistoryResource,
        TimezoneInterface $localeDate,
        StoreManagerInterface $storeManager
    ) {
        $this->priceHistoryResource = $priceHistoryResource;
        $this->localeDate = $localeDate;
        $this->storeManager = $storeManager;
    }

    public function get(int $productId, int $period, int $customerGroupId = 0, int $websiteId = null): float
    {
        if (!$websiteId) {
            $websiteId = (int)$this->storeManager->getWebsite()->getId();
        }

        $store = $this->storeManager->getWebsite($websiteId)->getDefaultStore();
        $dateObj = $this->localeDate->scopeDate($store, strtotime('-' . $period . ' day'), true);
        $datetime = $dateObj->format(DateTime::DATETIME_PHP_FORMAT);

        return $this->priceHistoryResource->getProductLowestPrice(
            $productId,
            $datetime,
            $websiteId,
            $customerGroupId
        );
    }
}
