<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel;

use Amasty\PriceHistory\Model\PriceHistory\PriceHistory as PriceHistoryModel;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\Data\Collection\Db\FetchStrategyInterface;
use Magento\Framework\Data\Collection\EntityFactoryInterface;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Model\ResourceModel\Db\AbstractDb;
use Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection;
use Psr\Log\LoggerInterface;

class Collection extends AbstractCollection
{
    /**
     * @var MetadataPool
     */
    private MetadataPool $metadataPool;

    public function __construct(
        EntityFactoryInterface $entityFactory,
        LoggerInterface $logger,
        FetchStrategyInterface $fetchStrategy,
        ManagerInterface $eventManager,
        MetadataPool $metadataPool,
        AdapterInterface $connection = null,
        AbstractDb $resource = null
    ) {
        parent::__construct($entityFactory, $logger, $fetchStrategy, $eventManager, $connection, $resource);
        $this->metadataPool = $metadataPool;
    }

    protected function _construct()
    {
        $this->_init(
            PriceHistoryModel::class,
            PriceHistory::class
        );
        $this->_setIdFieldName($this->getResource()->getIdFieldName());
    }

    public function joinSku(): self
    {
        $metadata = $this->metadataPool->getMetadata(ProductInterface::class);
        $entityField = $metadata->getIdentifierField();
        $entityTable = $metadata->getEntityTable();

        $this->getSelect()->joinInner(
            ['cpe' => $entityTable],
            'cpe.' . $entityField . ' = main_table.' . PriceHistoryModel::PRODUCT_ID,
            ['sku']
        );

        return $this;
    }
}
