<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\Utils\ParentIdsUpdater;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;

class FillLatestPriceTable
{
    public const LATEST_PRICE_TABLE_NAME = 'amasty_price_history_latest_price';

    /**
     * @var PriceHistory
     */
    private PriceHistory $priceHistoryResource;

    /**
     * @var ParentIdsUpdater
     */
    private ParentIdsUpdater $parentIdsUpdater;

    public function __construct(
        PriceHistory $priceHistoryResource,
        ParentIdsUpdater $parentIdsUpdater
    ) {
        $this->priceHistoryResource = $priceHistoryResource;
        $this->parentIdsUpdater = $parentIdsUpdater;
    }

    public function execute(?array $ids): void
    {
        $connection = $this->priceHistoryResource->getConnection();
        $latestPriceTable = $this->priceHistoryResource->getTable(self::LATEST_PRICE_TABLE_NAME);

        $select = $connection->select()
            ->from(
                $this->priceHistoryResource->getTable('catalog_product_index_price'),
                [
                    'product_id' => 'entity_id',
                    'customer_group_id',
                    'website_id',
                    'price' => $this->priceHistoryResource->getMinPriceExpr()
                ]
            );
        if (!empty($ids)) {
            $this->parentIdsUpdater->execute($ids);
            $select->where('entity_id IN (?)', $ids);
            $connection->delete($latestPriceTable, ['product_id IN (?)' => $ids]);
        } else {
            $connection->delete($latestPriceTable);
        }

        $connection->query($connection->insertFromSelect(
            $select,
            $latestPriceTable,
            ['product_id', 'customer_group_id', 'website_id', 'price']
        ));
    }
}
