<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\SelectModifiers;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\FillLatestPriceTable;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;
use Magento\Framework\DB\Select;

class AddLatestPrice implements SelectModifierInterface
{
    /**
     * @var PriceHistory
     */
    private PriceHistory $priceHistoryResource;

    public function __construct(
        PriceHistory $priceHistoryResource
    ) {
        $this->priceHistoryResource = $priceHistoryResource;
    }

    public function modify(Select $select): void
    {
        $select->joinLeft(
            ['lp' => $this->priceHistoryResource->getTable(FillLatestPriceTable::LATEST_PRICE_TABLE_NAME)],
            'e.entity_id = lp.product_id AND '
            . 'e.customer_group_id = lp.customer_group_id AND '
            . 'e.website_id = lp.website_id',
            'lp.price AS latest_price'
        );
    }
}
