<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\SelectModifiers;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\FillLatestPriceTable;
use Magento\Framework\DB\Select;

class FilterUnique implements SelectModifierInterface
{
    public function modify(Select $select): void
    {
        if ($this->validateLatestPriceTable($select)) {
            $select->having(new \Zend_Db_Expr('new_price != latest_price OR ISNULL(latest_price)'));
        }
    }

    private function validateLatestPriceTable(Select $select): bool
    {
        foreach ($select->getPart(Select::FROM) as $table) {
            if (isset($table['tableName'])
                && stripos($table['tableName'], FillLatestPriceTable::LATEST_PRICE_TABLE_NAME) !== false
            ) {
                return true;
            }
        }

        return false;
    }
}
