<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\SelectModifiers\SelectModifierInterface;
use Magento\Framework\ObjectManagerInterface;

class SelectModifiersProvider
{
    private const DEFAULT_SORT_ORDER = 0;

    /**
     * Update Select modifiers storage
     *
     * @var array
     */
    private array $modifiers = [];

    /**
     * @var ObjectManagerInterface
     */
    private ObjectManagerInterface $objectManager;

    /**
     * @param ObjectManagerInterface $objectManager
     * @param array $modifiers ['name' => ['class, 'sortOrder']]
     */
    public function __construct(
        ObjectManagerInterface $objectManager,
        array $modifiers = []
    ) {
        $this->objectManager = $objectManager;
        $this->initModifiers($modifiers);
    }

    /**
     * @return SelectModifierInterface[]
     */
    public function get(): array
    {
        return $this->modifiers;
    }

    private function initModifiers(array $modifiers): void
    {
        foreach ($modifiers as &$modifier) {
            if (!isset($modifier['class']) || !is_subclass_of($modifier['class'], SelectModifierInterface::class)) {
                throw new \LogicException(
                    sprintf('Modifier must implement %s', SelectModifierInterface::class)
                );
            }
            $modifier['sortOrder'] ??= self::DEFAULT_SORT_ORDER;
        }
        unset($modifier);

        uasort($modifiers, function ($first, $second) {
            return $first['sortOrder'] <=> $second['sortOrder'];
        });

        foreach ($modifiers as $key => $modifier) {
            $this->modifiers[$key] = $this->objectManager->get($modifier['class']);
        }
    }
}
