<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\Utils\ParentIdsUpdater;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;
use Magento\Framework\DB\Select;

class UpdatePriceHistoryTable
{
    /**
     * @var PriceHistory
     */
    private PriceHistory $priceHistoryResource;

    /**
     * @var SelectModifiersApplier
     */
    private SelectModifiersApplier $selectModifiersApplier;

    /**
     * @var ParentIdsUpdater
     */
    private ParentIdsUpdater $parentIdsUpdater;

    public function __construct(
        PriceHistory $priceHistoryResource,
        SelectModifiersApplier $selectModifiersApplier,
        ParentIdsUpdater $parentIdsUpdater
    ) {
        $this->priceHistoryResource = $priceHistoryResource;
        $this->selectModifiersApplier = $selectModifiersApplier;
        $this->parentIdsUpdater = $parentIdsUpdater;
    }

    public function execute(?array $ids): void
    {
        $connection = $this->priceHistoryResource->getConnection();
        $select = $connection->select();

        $select->from(
            ['e' => $this->priceHistoryResource->getTable('catalog_product_index_price')],
            [
                'product_id' => 'entity_id',
                'customer_group_id',
                'website_id',
                'new_price' => $this->priceHistoryResource->getMinPriceExpr()
            ]
        );
        $this->selectModifiersApplier->execute($select);
        if (!empty($ids)) {
            $this->parentIdsUpdater->execute($ids);
            $select->where('entity_id IN (?)', $ids);
        }

        $columns = [];
        foreach ($select->getPart(Select::COLUMNS) as $column) {
            $columns[] = $column[2] ?? $column[1];
        }
        $connection->query($connection->insertFromSelect(
            $select,
            $this->priceHistoryResource->getTable(PriceHistory::TABLE_NAME),
            $columns
        ));
    }
}
