<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\Utils;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\EntityManager\EntityMetadataInterface;
use Magento\Framework\EntityManager\MetadataPool;

/**
 * Even if index is executed by rows - price indexer also performs execution on parent products
 * We must do the same
 *
 * @see \Magento\Catalog\Model\Indexer\Product\Price\AbstractAction::_reindexRows
 */
class ParentIdsUpdater
{
    /**
     * @var PriceHistory
     */
    private PriceHistory $resource;

    /**
     * @var EntityMetadataInterface
     */
    private EntityMetadataInterface $productMetadata;

    public function __construct(
        PriceHistory $resource,
        MetadataPool $metadataPool
    ) {
        $this->resource = $resource;
        $this->productMetadata = $metadataPool->getMetadata(ProductInterface::class);
    }

    public function execute(array &$ids): void
    {
        if ($parentIds = $this->getParentProductsTypes($ids)) {
            $ids = array_merge($ids, $parentIds);
        }
    }

    private function getParentProductsTypes(array $productsIds): array
    {
        $select = $this->resource->getConnection()->select()->from(
            ['l' => $this->resource->getTable('catalog_product_relation')],
            ''
        )->join(
            ['e' => $this->resource->getTable('catalog_product_entity')],
            'e.' . $this->productMetadata->getLinkField() . ' = l.parent_id',
            ['e.entity_id as parent_id']
        )->where('l.child_id IN (?)', $productsIds);

        return (array)$this->resource->getConnection()->fetchCol($select);
    }
}
