<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Model\Webapi\Handlers;

use Amasty\PriceHistory\Api\Data\PriceDataInterfaceFactory;
use Amasty\PriceHistory\Model\ConfigProvider;
use Amasty\PriceHistory\Model\PriceHistory\ProductLowestPriceProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Customer\Model\Context as CustomerContext;
use Magento\Framework\App\Http\Context;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class ReadHandler implements ExtensionInterface
{
    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var ProductLowestPriceProvider
     */
    private ProductLowestPriceProvider $lowestPriceProvider;

    /**
     * @var Context
     */
    private Context $httpContext;

    /**
     * @var PriceCurrencyInterface
     */
    private PriceCurrencyInterface $priceCurrency;

    /**
     * @var PriceDataInterfaceFactory
     */
    private PriceDataInterfaceFactory $priceDataFactory;

    public function __construct(
        ConfigProvider $configProvider,
        ProductLowestPriceProvider $lowestPriceProvider,
        Context $httpContext,
        PriceCurrencyInterface $priceCurrency,
        PriceDataInterfaceFactory $priceDataFactory
    ) {
        $this->configProvider = $configProvider;
        $this->lowestPriceProvider = $lowestPriceProvider;
        $this->httpContext = $httpContext;
        $this->priceCurrency = $priceCurrency;
        $this->priceDataFactory = $priceDataFactory;
    }

    /**
     * @param Product $entity
     * @param array $arguments
     *
     * @return Product
     */
    public function execute($entity, $arguments = [])
    {
        if (!$price = $this->getPrice($entity)) {
            return $entity;
        }

        $storeId = (int)$entity->getStoreId();
        $period = $this->configProvider->getDisplayPeriod($storeId);
        $formattedPrice = $this->priceCurrency->convertAndFormat($price, false);
        $message = __('The lowest price from last %1 days: %2', $period, $formattedPrice)->render();

        $priceData = $this->priceDataFactory->create();
        $priceData->setLowestPrice($formattedPrice);
        $priceData->setLowestPriceMessage($message);
        $entity->getExtensionAttributes()->setAmOmnibusPrice($priceData);

        return $entity;
    }

    private function getPrice(ProductInterface $entity): float
    {
        $productId = (int)$entity->getEntityId();
        $storeId = (int)$entity->getStoreId();
        $period = $this->configProvider->getDisplayPeriod($storeId);
        $customerGroupId = (int)$this->httpContext->getValue(CustomerContext::CONTEXT_GROUP);
        $websiteId = (int)$entity->getStore()->getWebsite()->getId();

        return $this->lowestPriceProvider->get($productId, $period, $customerGroupId, $websiteId);
    }
}
