<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Plugin\Catalog\Block\Product\ListProduct;

use Amasty\PriceHistory\Model\ConfigProvider;
use Amasty\PriceHistory\Model\Frontend\LowestPriceRenderer;
use Amasty\PriceHistory\Model\OptionSource\DisplayPrices;
use Magento\Catalog\Block\Product\ListProduct;
use Magento\Catalog\Model\Product;

class AddLowestPrices
{
    public const CATEGORY_PAGE_CLASS = 'category';

    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var LowestPriceRenderer
     */
    private LowestPriceRenderer $lowestPriceRenderer;

    public function __construct(
        ConfigProvider $configProvider,
        LowestPriceRenderer $lowestPriceRenderer
    ) {
        $this->configProvider = $configProvider;
        $this->lowestPriceRenderer = $lowestPriceRenderer;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param ListProduct $subject
     * @param string $html
     * @param Product $product
     *
     * @return string
     */
    public function afterGetProductPrice(ListProduct $subject, string $html, Product $product): string
    {
        if ($this->configProvider->getDisplayPrices() === DisplayPrices::PRODUCT_AND_CATEGORY) {
            $html .= $this->lowestPriceRenderer->render($product, self::CATEGORY_PAGE_CLASS);
        }

        return $html;
    }
}
