<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Plugin\Catalog\Model\Indexer\Product\Price;

use Amasty\PriceHistory\Model\ConfigProvider;
use Amasty\PriceHistory\Model\PriceHistory\PriceHistoryProcessorFactory;
use Amasty\PriceHistory\Model\PriceHistory\Processor\PriceHistoryProcessorInterface;
use Magento\Catalog\Model\Indexer\Product\Price\AbstractAction;

class SavePriceChanges
{
    /**
     * @var PriceHistoryProcessorInterface
     */
    private PriceHistoryProcessorInterface $processor;

    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    public function __construct(
        PriceHistoryProcessorFactory $priceHistoryProcessorFactory,
        ConfigProvider $configProvider
    ) {
        $this->processor = $priceHistoryProcessorFactory->create(PriceHistoryProcessorFactory::INDEXER_PROCESSOR);
        $this->configProvider = $configProvider;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param AbstractAction $subject
     * @param array|int|null $ids
     *
     * @return array|null
     */
    public function beforeExecute(AbstractAction $subject, $ids = null): ?array
    {
        if ($this->configProvider->isEnabled()) {
            if (is_numeric($ids)) {
                $this->processor->initialize([$ids]); //1 row reindex
            } else {
                $this->processor->initialize($ids);
            }
        }

        return $ids !== null ? [$ids] : $ids;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param AbstractAction $subject
     * @param null $result,
     * @param array|int|null $ids
     *
     * @return void
     */
    public function afterExecute(AbstractAction $subject, $result, $ids = null): void
    {
        if ($this->configProvider->isEnabled()) {
            if (is_numeric($ids)) {
                $this->processor->execute([$ids]); //1 row reindex
            } else {
                $this->processor->execute($ids);
            }
        }
    }
}
