<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Setup\Patch\Data;

use Amasty\PriceHistory\Model\PriceHistory\PriceHistory as PriceHistoryModel;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory;
use Magento\Framework\Setup\Patch\DataPatchInterface;

class FillHistoryTable implements DataPatchInterface
{
    /**
     * @var PriceHistory
     */
    private PriceHistory $priceHistoryResource;

    public function __construct(
        PriceHistory $priceHistoryResource
    ) {
        $this->priceHistoryResource = $priceHistoryResource;
    }

    public static function getDependencies(): array
    {
        return [];
    }

    public function getAliases(): array
    {
        return [];
    }

    public function apply()
    {
        $connection = $this->priceHistoryResource->getConnection();
        $indexTable = $this->priceHistoryResource->getTable('catalog_product_index_price');

        $select = $connection->select();
        $select->from(
            ['e' => $indexTable],
            [
                'product_id' => 'entity_id',
                'customer_group_id',
                'website_id',
                'new_price' => $this->priceHistoryResource->getMinPriceExpr()]
        );

        $connection->query($connection->insertFromSelect(
            $select,
            $this->priceHistoryResource->getTable(PriceHistory::TABLE_NAME),
            [
                PriceHistoryModel::PRODUCT_ID,
                PriceHistoryModel::CUSTOMER_GROUP_ID,
                PriceHistoryModel::WEBSITE_ID,
                PriceHistoryModel::NEW_PRICE
            ]
        ));

        return $this;
    }
}
