<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Test\Integration\Model\Frontend\DisplayFilter;

use Amasty\PriceHistory\Model\Frontend\DisplayFilter\OnSaleFilter;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\TestFramework\Helper\Bootstrap;
use PHPUnit\Framework\TestCase;

/**
 * @magentoDbIsolation disabled
 * @magentoAppIsolation enabled
 */
class OnSaleFilterTest extends TestCase
{
    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    protected function setUp(): void
    {
        $this->objectManager = Bootstrap::getObjectManager();
        $this->productRepository = $this->objectManager->get(ProductRepositoryInterface::class);
    }

    /**
     * @magentoDataFixture Magento/Catalog/_files/product_simple.php
     *
     * @magentoConfigFixture current_store price_history/general/display_mode 1
     */
    public function testShowSimpleProduct(): void
    {
        $product = $this->productRepository->get('simple');

        $filter = $this->objectManager->get(OnSaleFilter::class);
        $result = $filter->show($product);
        $this->assertFalse($result);
    }

    /**
     * @magentoDataFixture Magento/Catalog/_files/product_special_price.php
     *
     * @magentoConfigFixture current_store price_history/general/display_mode 1
     */
    public function testShowSpecialPriceProduct(): void
    {
        $product = $this->productRepository->get('simple');

        $filter = $this->objectManager->get(OnSaleFilter::class);
        $result = $filter->show($product);
        $this->assertTrue($result);
    }

    /**
     * @magentoDataFixture Magento/Catalog/_files/product_simple.php
     * @magentoDataFixture Magento/CatalogRule/_files/catalog_rule_25_customer_group_all.php
     * @magentoDataFixture Amasty_PriceHistory::Test/Integration/_files/catalog_product_price_reindex_all.php
     *
     * @magentoConfigFixture current_store price_history/general/display_mode 1
     */
    public function testShowCatalogRule(): void
    {
        $product = $this->productRepository->get('simple');

        $filter = $this->objectManager->get(OnSaleFilter::class);
        $result = $filter->show($product);
        $this->assertTrue($result);
    }

    /**
     * @magentoDataFixture Magento/Bundle/_files/product.php
     *
     * @magentoConfigFixture current_store price_history/general/display_mode 1
     */
    public function testShowBundleProduct(): void
    {
        $product = $this->productRepository->get('bundle-product');

        $filter = $this->objectManager->get(OnSaleFilter::class);
        $result = $filter->show($product);
        $this->assertFalse($result);
    }
}
