<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Test\Integration\Model\PriceHistory;

use Amasty\PriceHistory\Model\PriceHistory\ProductLowestPriceProvider;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\PriceHistory as PriceHistoryResource;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\TestFramework\Helper\Bootstrap;
use PHPUnit\Framework\TestCase;

class ProductLowestPriceProviderTest extends TestCase
{
    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;

    protected function setUp(): void
    {
        $this->objectManager = Bootstrap::getObjectManager();
    }

    protected function tearDown(): void
    {
        $this->objectManager->get(PriceHistoryResource::class)->clean();
    }

    /**
     * @magentoDbIsolation disabled
     * @magentoAppIsolation enabled
     *
     * @magentoDataFixture Amasty_PriceHistory::Test/Integration/_files/fill_price_history_table.php
     * @magentoDataFixture Amasty_PriceHistory::Test/Integration/_files/catalog_product_price_reindex_all.php
     *
     * @dataProvider recordSearchDataProvider
     */
    public function testRecordSearch(int $period, int $expected, $prodIdIncr = 0): void
    {
        $provider = $this->objectManager->get(ProductLowestPriceProvider::class);

        $productRepository = $this->objectManager->get(ProductRepositoryInterface::class);
        $product = $productRepository->get('simple');

        $value = $provider->get((int)$product->getId() + $prodIdIncr, $period);
        $this->assertEquals($value, $expected);
    }

    public function recordSearchDataProvider(): array
    {
        return [
            '10 days period' => [10, 5],
            '5 days period' => [5, 10],
            'not indexed product' => [10, 0, 1],
            'product out of period' => [1, 0]
        ];
    }
}
