<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Test\Unit\Model\Frontend;

use Amasty\PriceHistory\Model\Frontend\DisplayFilter\FilterInterface;
use Amasty\PriceHistory\Model\Frontend\DisplayFilterPool;
use Amasty\PriceHistory\Model\Frontend\DisplayValidator;
use Magento\Catalog\Api\Data\ProductInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

class DisplayValidatorTest extends TestCase
{
    /**
     * @covers DisplayValidator::validate
     * @dataProvider validateDataProvider
     */
    public function testValidate(array $filters, bool $expected): void
    {
        $pool = $this->createMock(DisplayFilterPool::class);
        $pool->expects($this->any())->method('getAll')->willReturn($filters);

        $validator = new DisplayValidator($pool);
        $result = $validator->validate($this->createMock(ProductInterface::class));
        $this->assertEquals($result, $expected);
    }

    private function validateDataProvider(): array
    {
        return [
            'no filters' => [[], true],
            'all valid filters' => [
                [
                    $this->createDisplayFilterMock(true),
                    $this->createDisplayFilterMock(true)
                ],
                true
            ],
            '2nd filter invalid' => [
                [
                    $this->createDisplayFilterMock(true),
                    $this->createDisplayFilterMock(false)
                ],
                false
            ],
            '1st filter invalid' => [
                [
                    $this->createDisplayFilterMock(false),
                    $this->createDisplayFilterMock(true, false)
                ],
                false
            ]
        ];
    }

    private function createDisplayFilterMock(bool $returnResult, bool $willBeCalled = true): MockObject
    {
        $filterMock = $this->createMock(FilterInterface::class);

        $filterMock->expects($willBeCalled ? $this->once() : $this->never())
            ->method('show')
            ->willReturn($returnResult);

        return $filterMock;
    }
}
