<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Omnibus Price Tracker by Amasty
 */

namespace Amasty\PriceHistory\Test\Unit\Model\PriceHistory\ResourceModel\IndexProcessorActions;

use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\SelectModifiersProvider;
use Magento\Framework\ObjectManagerInterface;
use PHPUnit\Framework\TestCase;
use Amasty\PriceHistory\Model\PriceHistory\ResourceModel\IndexProcessorActions\SelectModifiers\SelectModifierInterface;

class SelectModifiersProviderTest extends TestCase
{
    /**
     * @covers SelectModifiersProvider::initModifiers
     * @dataProvider initializationDataProvider
     */
    public function testInitialization(array $modifiersConfig, array $modifiersOrder): void
    {
        $omMock = $this->createMock(ObjectManagerInterface::class);
        $provider = new SelectModifiersProvider($omMock, $modifiersConfig);
        $modifiers = $provider->get();

        $this->assertEquals($modifiersOrder, array_keys($modifiers));
    }

    public function testWrongClassInitialization(): void
    {
        $modifiersConfig = [
            ['class' => 'Wrong/Class']
        ];
        $omMock = $this->createMock(ObjectManagerInterface::class);

        $this->expectException(\LogicException::class);
        new SelectModifiersProvider($omMock, $modifiersConfig);
    }

    private function initializationDataProvider(): array
    {
        $modifierMock = $this->createMock(SelectModifierInterface::class);
        $className = get_class($modifierMock);

        $modifier1 = ['class' => $className, 'sortOrder' => 10];
        $modifier2 = ['class' => $className, 'sortOrder' => 20];
        $modifierNoOrder = ['class' => $className];

        return [
            'no modifiers' => [[], []],
            'correct order' => [
                ['first' => $modifier1, 'second' => $modifier2],
                ['first', 'second']
            ],
            'wrong order' => [
                ['first' => $modifier2, 'second' => $modifier1],
                ['second', 'first']
            ],
            'correct order without sortOrder' => [
                ['first' => $modifierNoOrder, 'second' => $modifier1, 'third' => $modifier2],
                ['first', 'second', 'third']
            ],
            'wrong order without sortOrder' => [
                ['first' => $modifier2, 'second' => $modifierNoOrder, 'third' => $modifier1],
                ['second', 'third', 'first']
            ]
        ];
    }
}
